@@ -3,5 +3,6 @@ echo "Content-Type: text/plain"
 echo ""
 
 echo "hello, world."
-sleep 35
+pwd
+env
 
